HideFrames = CreateFrame("Frame", UIParent)

VIEWED_SPELL_BOOK = 2;
VIEWED_SPELL_BOOK_NUM_SPELLS = nil;
SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT = false
SPELL_BOOK_TYPE = "spells"
CURRENT_FLYOUT = nil

VIEWED_TAB_ID = nil

SPELL_BOOK_PROFESSIONS = {}

function BobUI_FormatProfession(PROFESSION_INDEX, PROFESSION_ID)
	if PROFESSION_ID == nil then SPELL_BOOK_PROFESSIONS[PROFESSION_INDEX] = nil; return end 
	local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset, skillLineName = GetProfessionInfo(PROFESSION_ID);
	
	SPELL_BOOK_PROFESSIONS[PROFESSION_INDEX] = {
		["name"] = name,
		["texture"] = texture,
		["rank"] = rank,
		["maxRank"] = maxRank,
		["numSpells"] = numSpells,
		["spelloffset"] = spelloffset,
		["skillLine"] = skillLine,
		["rankModifier"] = rankModifier,
		["specializationIndex"] = specializationIndex,
		["specializationOffset"] = specializationOffset,
		["skillLineName"] = skillLineName
	}

	if (PROFESSION_INDEX == 1) then
		BobUI_specButtons.specButton7.specIcon:SetTexture(texture);
	end
end


function GetNumSpellsForSpellBook(SPELL_BOOK)
	
	local name, texture, offset, numSlots, isGuild, offSpecID, shouldHide, specID = GetSpellTabInfo(SPELL_BOOK);

	if SPELL_BOOK == 6 then
		numSlots = select(1, HasPetSpells())
	end

	if SPELL_BOOK == 7 then -- professions
		local prof1, prof2, arch, fish, cook = GetProfessions();

		BobUI_FormatProfession(1, prof1);
		BobUI_FormatProfession(2, prof2);
		BobUI_FormatProfession(3, cook);
		BobUI_FormatProfession(4, fish);
		BobUI_FormatProfession(5, arch);
		
		numSlots = 0

		for i=1,5 do
			if (SPELL_BOOK_PROFESSIONS[i] ~= nil) then
				numSlots = numSlots + SPELL_BOOK_PROFESSIONS[i]["numSpells"]
			end
		end

	end


	return numSlots
end


function modifiedClick(spellIndex, button)
	if ( IsModifiedClick() ) then
		if ( IsModifiedClick("CHATLINK") ) then
			if ( MacroFrameText and MacroFrameText:HasFocus() ) then
				local spellName, subSpellName = GetSpellBookItemName(spellIndex, SPELL_BOOK_TYPE);
				if ( spellName and not IsPassiveSpell(spellIndex, SPELL_BOOK_TYPE) ) then
					if ( subSpellName and (strlen(subSpellName) > 0) ) then
						ChatEdit_InsertLink(spellName.."("..subSpellName..")");
					else
						ChatEdit_InsertLink(spellName);
					end
				end
				return;
			else
				local tradeSkillLink, tradeSkillSpellID = GetSpellTradeSkillLink(spellIndex, SPELL_BOOK_TYPE);
				if ( tradeSkillSpellID ) then
					ChatEdit_InsertLink(tradeSkillLink);
				else
					local spellLink = GetSpellLink(spellIndex, SPELL_BOOK_TYPE);
					ChatEdit_InsertLink(spellLink);
				end
				return;
			end
		end
	end
end

function getProfessionOffset(li)
	local returned = 0
	local numSpells = 0
	local spellCount = 0
	local isFirstOfProfession = false
	for i=1,5 do 
		if SPELL_BOOK_PROFESSIONS[i] ~= nil then
			spellCount = spellCount + SPELL_BOOK_PROFESSIONS[i]["numSpells"]
			if li < spellCount then 
				isFirstOfProfession = true
				returned = SPELL_BOOK_PROFESSIONS[i]["spelloffset"]
				numSpells = SPELL_BOOK_PROFESSIONS[i]["numSpells"]
				break
			elseif li == spellCount then 
				returned = SPELL_BOOK_PROFESSIONS[i]["spelloffset"] + 1
				numSpells = SPELL_BOOK_PROFESSIONS[i]["numSpells"]

				if numSpells == 1 then 
					isFirstOfProfession = true
				end

				break
			end
		end
	end

	return returned - li + (numSpells - 1), isFirstOfProfession
end

function UpdateCooldown(self, slot, slotType)
	
	if (slot) then

		local start, duration, enable, modRate = GetSpellCooldown(slot, slotType);

		if (self.cooldown and start and duration) then
			if (enable) then
				self.cooldown:Hide();
				self.cooldown.coolDownTimer:Hide();
			else
				self.cooldown:Show();
				self.cooldown.coolDownTimer:SetText((duration + start) - GetTime())
				self.cooldown.coolDownTimer:Show();
			end

			CooldownFrame_Set(self.cooldown, start, duration, enable, true, modRate);
		else
			self.cooldown:Hide();
			self.cooldown.coolDownTimer:Hide();
		end
	end
end


function BobUI_spellBookItemData(i)
	local offset = 0;
	local numFlyoutSpells = 0
			
	if SPELL_BOOK_TYPE == "profession" then
		offset, isFirstOfProfession = getProfessionOffset(i)
	elseif SPELL_BOOK_TYPE == "spell" then
		_, _, offset = GetSpellTabInfo(VIEWED_SPELL_BOOK);
	end

	local texture = GetSpellBookItemTexture(i+offset, SPELL_BOOK_TYPE);
	local spellType, actionID = GetSpellBookItemInfo(i+offset, SPELL_BOOK_TYPE);
	local spellName, subSpellName, spellID = GetSpellBookItemName(i+offset, SPELL_BOOK_TYPE)

	local isOnActionbar = (VIEWED_SPELL_BOOK ~= 2 and true or C_ActionBar.IsOnBarOrSpecialBar(actionID))
	if (SPELL_BOOK_TYPE == "pet") then
		isOnActionbar = C_ActionBar.HasPetActionButtons(actionID) or C_ActionBar.HasPetActionPetBarIndices(actionID);
	end

	local isPassive
	if (SPELL_BOOK_TYPE == "profession") then 
		isPassive = false 
	else 
		isPassive = IsPassiveSpell(i+offset, SPELL_BOOK_TYPE) 
	end

	if spellType == "FLYOUT" then
		numFlyoutSpells = select(3, GetFlyoutInfo(actionID))
	end
	local spellLevel = GetSpellAvailableLevel(i+offset, SPELL_BOOK_TYPE)

	local autoCastAllowed, autoCastEnabled = GetSpellAutocast(i+offset, SPELL_BOOK_TYPE)

	return offset, isFirstOfProfession, isPassive, isOnActionbar, texture, spellType, numFlyoutSpells, actionID, spellName, subSpellName, spellID, spellLevel, autoCastAllowed, autoCastEnabled
end


function AssignScriptsToButtons(button, i, offset, missingOnActionBar, SPELL_BOOK_TYPE, spellID, spellName, actionID, autoCastAllowed)
	button:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	button:SetScript("OnEnter", function()
		GameTooltip:Hide()
		GameTooltip:ClearAllPoints()
		GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
		
		if spellID then
			GameTooltip:SetSpellByID(spellID)
		else
			GameTooltip:SetSpellBookItem(i+offset, SPELL_BOOK_TYPE)
		end

		GameTooltip:AddLine(missingOnActionBar and SPELLBOOK_SPELL_NOT_ON_ACTION_BAR or "",LIGHTBLUE_FONT_COLOR.r, LIGHTBLUE_FONT_COLOR.g, LIGHTBLUE_FONT_COLOR.b);
		GameTooltip:Show()
	end)

	button:SetScript("OnDragStart", function()
		if spellID and SPELL_BOOK_TYPE ~= "pet" then
			if spellID == 53271 or spellID == 62305 or spellID == 272682 or spellID == 264735 or spellID == 281195 or spellID == 264667 or spellID == 272679 then
				PickupSpell(272651)
			else
				PickupSpell(spellID)
			end
		else
			if not IsPassiveSpell(i+offset, SPELL_BOOK_TYPE) then
				PickupSpellBookItem(i+offset, SPELL_BOOK_TYPE)
			end
		end
	end)

	button:RegisterEvent("SPELLS_CHANGED")
	button:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	button:SetScript("OnEvent", function(self, event, ...)
		if event == "SPELL_UPDATE_COOLDOWN" or event == "SPELLS_CHANGED" then
			UpdateCooldown(self, i+offset, SPELL_BOOK_TYPE)
		end
	end)

	if SPELL_BOOK_TYPE == "pet" then 
		if (spellID ~= nil) then
			if autoCastAllowed then
				button:RegisterForClicks("AnyUp")
				button:SetAttribute("type2", "macro");
				button:SetAttribute("macrotext", "/petautocasttoggle "..spellName);
			end
				button:SetAttribute("type1", "spell");
				button:SetAttribute("spell", spellID);
		else
			if spellName == "Defensive" then
				button:SetAttribute("type1", "macro");
				button:SetAttribute("macrotext", "/petdefensive"); -- doesn't work, idk how to fix.
			elseif spellName == "Move To" then
				button:SetAttribute("type1", "macro");
				button:SetAttribute("macrotext", "/petmoveto");
			else
				
				button:SetAttribute("type1", "macro");
				button:SetAttribute("macrotext", "/pet"..spellName);
			end
				
		end
	else 
		button:SetAttribute("type1", "spell");
		button:SetAttribute("spell", actionID);
		button:SetAttribute("target", "target");
	end
end



function toggleAutocastShine(button, bIndex)
	local autoCastableTexture = _G[button:GetName().."AutoCastable"];
	local petAutoCastShine = _G[button:GetName().."Shine"];
	local autoCastAllowed, autoCastEnabled = GetSpellAutocast(bIndex, SPELL_BOOK_TYPE)


	if ( autoCastAllowed ) then
		autoCastableTexture:Show();
	else
		autoCastableTexture:Hide();
	end
	
	if autoCastEnabled then
		AutoCastShine_AutoCastStart(petAutoCastShine)
	else
		AutoCastShine_AutoCastStop(petAutoCastShine)
	end
end

function updateSpellButtons()

	if VIEWED_SPELL_BOOK == 6 then 
		SPELL_BOOK_TYPE = "pet"
	elseif VIEWED_SPELL_BOOK == 7 then 
		SPELL_BOOK_TYPE = "profession"
	else
		SPELL_BOOK_TYPE = "spell"
	end



	local buttons;
	local newLineAt = 9;

	if ( BobTabPage1.buttons ) then
		buttons = BobTabPage1.buttons;
	else
		return
	end


	local j = 1
	local k = 1
	local l = 1
	local m = 1
	local vCi= 1
	local unlearnProfession = 0

	local startOfLinePassives = nil
	local startOfLineActives = nil
	local startOfLinePetControls = nil
	local startOfLineFlyouts = nil

	local lastPassive = nil
	local lastActive = nil
	local lastPetControl = nil
	local lastFlyout = nil

	local LastFirstOfProfession = nil

	BobTabPage1Passives:SetHeight(0)
	BobTabPage1Actives:SetHeight(0)
	BobTabPage1PetControlsAndFlyouts:SetHeight(0)
	BobTabPage1FlyoutButtons:SetHeight(40)
	
	for i = 1, #buttons do -- attempt to fix weird bug that some people experience, but I'm unable to recreate
		local button = buttons[i];
		button:Hide(); 
	end

	for i = 1, #buttons do 
		local button = buttons[i];
		if i > VIEWED_SPELL_BOOK_NUM_SPELLS then
			--button:Hide(); 
		else
			local offset, isFirstOfProfession, isPassive, isOnActionbar, texture, spellType, numFlyoutSpells, actionID, spellName, subSpellName, spellID, spellLevel, autoCastAllowed, autoCastEnabled = BobUI_spellBookItemData(i)

			toggleAutocastShine(button, i+offset)

			if (spellType == "FLYOUT") then
				button.FlyoutArrow:Show()
				SetClampedTextureRotation(button.FlyoutArrow, 180);
				isOnActionbar = true
			else
				button.FlyoutArrow:Hide()
			end

			local missingOnActionBar = ((isOnActionbar ~= true) and (isPassive ~= true))
			
			local isDisabled = spellID and C_SpellBook.IsSpellDisabled(spellID);

			if not (spellType == "FUTURESPELL") and not isDisabled then
					button.TrainBook:Hide()
					_G[button:GetName().."IconTexture"]:SetDesaturated(false)
					_G[button:GetName().."RequiredLevelString"]:Hide()				
			else
				if spellType == "FUTURESPELL" then
					missingOnActionBar = false

					if (spellLevel and spellLevel > UnitLevel("player") or isDisabled) then
						_G[button:GetName().."RequiredLevelString"]:SetText("Lv "..spellLevel)
						_G[button:GetName().."RequiredLevelString"]:Show()
					else
						button.TrainBook:Show()
					end

					_G[button:GetName().."IconTexture"]:SetDesaturated(true)
				end
			end
			if missingOnActionBar then button.SpellHighlightTexture:Show() else button.SpellHighlightTexture:Hide() end

			local newLinePassives = (math.fmod(j-1,newLineAt) == 0)
			local newLineActives = (math.fmod(k-1,newLineAt) == 0)
			local newLinePetControls = (math.fmod(l-1,newLineAt) == 0)
			local newLineFlyouts = (math.fmod(m-1,newLineAt) == 0)

			button:ClearAllPoints()
			if spellType == "FLYOUT" then
				if m - 1 == 0 then
					button:SetPoint("TOPLEFT", BobTabPage1PetControlsAndFlyouts, "TOPLEFT", 0, 0);
				else
					if (newLineFlyouts) then
						button:SetPoint("TOPLEFT", startOfLineFlyouts, "BOTTOMLEFT", 0,-4);
					end
				end
				if newLineFlyouts then
					startOfLineFlyouts = buttons[i]
					BobTabPage1PetControlsAndFlyouts:SetHeight(BobTabPage1PetControlsAndFlyouts:GetHeight() + 38)
				else
					button:SetPoint("LEFT", lastFlyout or startOfLineFlyouts, "RIGHT", 4, 0);
				end
				lastFlyout = buttons[i]

			else
				if isPassive then

					if j - 1 == 0 then
						button:SetPoint("TOPLEFT", BobTabPage1Passives, "TOPLEFT", 0, 0);
					else
						if (newLinePassives) then
							button:SetPoint("TOPLEFT", startOfLinePassives, "BOTTOMLEFT", 0,-4);
						end
					end
					if newLinePassives then
						startOfLinePassives = buttons[i]
						BobTabPage1Passives:SetHeight(BobTabPage1Passives:GetHeight() + 38)
					else
						button:SetPoint("LEFT", lastPassive or startOfLinePassives, "RIGHT", 4, 0);
					end
					lastPassive = buttons[i]

				else
					if spellID == nil and SPELL_BOOK_TYPE == "pet" then
						if l - 1 == 0 then
							button:SetPoint("TOPLEFT", BobTabPage1PetControlsAndFlyouts, "TOPLEFT", 0, 0);
						else
							if (newLinePetControls) then
								button:SetPoint("TOPLEFT", startOfLinePetControls, "BOTTOMLEFT", 0,-4);
							end
						end
						if newLinePetControls then
							startOfLinePetControls = buttons[i]
							BobTabPage1PetControlsAndFlyouts:SetHeight(BobTabPage1PetControlsAndFlyouts:GetHeight() + 38)
						else
							button:SetPoint("LEFT", lastPetControl or startOfLinePetControls, "RIGHT", 4, 0);
						end
						lastPetControl = buttons[i]
					else
						if k - 1 == 0 then
							button:SetPoint("TOPLEFT", BobTabPage1Actives, "TOPLEFT", 0, 0);
						else
							if (newLineActives) then
								button:SetPoint("TOPLEFT", startOfLineActives, "BOTTOMLEFT", 0,-4);
							end
						end
						if newLineActives then
							startOfLineActives = buttons[i]
							BobTabPage1Actives:SetHeight(BobTabPage1Actives:GetHeight() + 38)
						else
							button:SetPoint("LEFT", lastActive or startOfLineActives, "RIGHT", 4, 0);
						end
						lastActive = buttons[i]
					end

				end
			end

			AssignScriptsToButtons(button, i, offset, missingOnActionBar, SPELL_BOOK_TYPE, spellID, spellName, actionID, autoCastAllowed)
			
			button:SetScript("OnMouseUp", function(self, mBtn)
				if ( IsModifiedClick() ) then
					modifiedClick(i+offset, mBtn)
				else
					if (spellType == "FLYOUT") then -----------
						local startPt = VIEWED_SPELL_BOOK_NUM_SPELLS + 1
						local previousButton = nil
						local newLineBtn = nil
						local btnCount = 1
						if CURRENT_FLYOUT == actionID then
							for x1 = startPt, #buttons do 
								buttons[x1]:Hide()
							end
							CURRENT_FLYOUT = nil
						else
							CURRENT_FLYOUT = actionID

							for x1 = startPt, #buttons do 
								local flyoutButton = buttons[x1]
								flyoutButton:ClearAllPoints()

								if x1 > startPt + numFlyoutSpells - 1 then
									flyoutButton:Hide()
								else

									local flyoutSpellID, flyoutOverrideSpellID, flyoutIsKnown, flyoutSpellName, flyoutSlotSpecID = GetFlyoutSlotInfo(actionID, (x1-startPt)+1);
									

									if flyoutIsKnown then
										if previousButton == nil then
											flyoutButton:SetPoint("TOPLEFT", BobTabPage1FlyoutButtons, "TOPLEFT", 0, 0);
											newLineBtn = flyoutButton
										elseif btnCount == 8 then
											flyoutButton:SetPoint("TOPLEFT", newLineBtn, "BOTTOMLEFT", 0, -2);
											btnCount = 1
											newLineBtn = flyoutButton
										else
											flyoutButton:SetPoint("LEFT", previousButton, "RIGHT", 2, 0);
										end
										
										AssignScriptsToButtons(flyoutButton, 0, 0, false, nil, flyoutSpellID, flyoutSpellName, flyoutSpellID, autoCastAllowed)

										previousButton = buttons[x1]
										btnCount = btnCount + 1

										_G[flyoutButton:GetName().."IconTexture"]:SetTexture(GetSpellBookItemTexture(flyoutSpellName))
										_G[flyoutButton:GetName().."IconTexture"]:Show()
										flyoutButton:SetSize(30,30)
										flyoutButton:Show()
									else
										flyoutButton:Hide()
									end
								end
							end
						end
					elseif (mBtn == "RightButton" and SPELL_BOOK_TYPE == "pet") then
						
						wait(0.2, toggleAutocastShine, button, i+offset)
						updateSpellButtons()
					end
				end
			end)
			

			
			
			_G[button:GetName().."IconTexture"]:SetTexture(texture)
			_G[button:GetName().."IconTexture"]:Show()
			button.cooldown:Hide();
			button.cooldown.coolDownTimer:Hide();
			button.unlearn:Hide()
			button.profText:Hide()
			button:SetSize(36,36)




			if SPELL_BOOK_TYPE == "profession" then
				if isFirstOfProfession then
					unlearnProfession = unlearnProfession + 1
					local vCi2 = unlearnProfession

					if (LastFirstOfProfession == nil) then
						button:SetPoint("TOPLEFT", BobTabPage1Actives, "TOPLEFT", 0, 0);
					else
						button:SetPoint("TOPLEFT", LastFirstOfProfession, "BOTTOMLEFT", 0, -4);
					end

					button:SetSize(48,48)
					button.profText:SetText(SPELL_BOOK_PROFESSIONS[vCi]["name"] .. "\n" .. SPELL_BOOK_PROFESSIONS[vCi]["rank"] .. "/" .. SPELL_BOOK_PROFESSIONS[vCi]["maxRank"])
					button.profText:SetPoint("LEFT", button, "RIGHT", 46, 0);
					button.profText:Show()
					button.unlearn:SetScript("OnClick", function()
						StaticPopup_Show("UNLEARN_SKILL", SPELL_BOOK_PROFESSIONS[vCi2]["name"], nil, SPELL_BOOK_PROFESSIONS[vCi2]["skillLine"]);
					end)
				end

				if ((LastFirstOfProfession == buttons[1] or LastFirstOfProfession == nil) and isFirstOfProfession) then -- need to test if this is always the case
					if (i+offset >= 0 ) then
						button.unlearn:Show()
					end
				end
				
				if isFirstOfProfession then
					LastFirstOfProfession = buttons[i]
					vCi = vCi + 1
				end
			end

			button:Show(); 
			

			if isPassive then j = j + 1; elseif spellID == nil and SPELL_BOOK_TYPE == "pet" then l = l + 1 elseif spellType == "FLYOUT" then m = m + 1 else k = k + 1 end
		end
	end
end

local waitTable = {};
local waitFrame = nil;

function wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

function createSpellButtons()
	local buttons, button;
	local numFlyoutSpells = 0

	if ( BobTabPage1.buttons ) then
		buttons = BobTabPage1.buttons;
	else
		buttons = {}
	end

	local _, _, offset = GetSpellTabInfo(VIEWED_SPELL_BOOK);

	-- creates flyout buttons
	for i = 1, VIEWED_SPELL_BOOK_NUM_SPELLS do
		local spellType, actionID = GetSpellBookItemInfo(i+offset, SPELL_BOOK_TYPE);
		if spellType == "FLYOUT" then
			local numCurrentFlyoutSpells = select(3, GetFlyoutInfo(actionID))

			if numFlyoutSpells < numCurrentFlyoutSpells then
				numFlyoutSpells = numCurrentFlyoutSpells
			end
		end
	end

	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);

	local numButtonsToCreate = 0

	for i=2, numSpecs+1 do
		if GetNumSpellsForSpellBook(i) > numButtonsToCreate then numButtonsToCreate = GetNumSpellsForSpellBook(i) end
	end

	-- create spell buttons
	for i = #buttons + 1, numButtonsToCreate + numFlyoutSpells do 

		button = CreateFrame("Button", "abilityButton"..i, BobTabPage1, "BobUI_SpellButtonTemplate,SecureActionButtonTemplate");
		button:SetSize(32,32)
		button:RegisterForDrag("LeftButton");

		_G[button:GetName().."RequiredLevelString"]:SetFont("Fonts\\ARIALN.ttf", 12, "OUTLINE")
		_G[button:GetName().."RequiredLevelString"]:SetTextColor(0.8,0.8,0.8,1)

		tinsert(buttons, button)
	end
	
	BobTabPage1.buttons = buttons
end

function specButtonsUpdateBorders()

	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);
	local currentSpecID = GetSpecialization()
	
	for i=1,8 do
		local BobUIButton = BobUI_specButtons["specButton"..i];



		if (i == currentSpecID+1) then
			BobUIButton:SetBackdrop({ edgeFile = "Interface/Buttons/WHITE8x8", tile = true, tileEdge = true, tileSize = 2, edgeSize = 2});
			BobUIButton:SetBackdropBorderColor(0.1, 1, 0.1, 1)
		else
			if BobUIButton:GetID() == VIEWED_TAB_ID then
				BobUIButton:SetBackdrop({ edgeFile = "Interface/Buttons/WHITE8x8", tile = true, tileEdge = true, tileSize = 2, edgeSize = 2});
				BobUIButton:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
			else
				BobUIButton:SetBackdrop({ edgeFile = "Interface/Buttons/WHITE8x8", tile = true, tileEdge = true, tileSize = 2, edgeSize = 2});
				BobUIButton:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
			end
		end
	end
end

function BobUI_TalentFrameSpec_OnShow(self)
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
end
function BobUI_TalentFrameSpec_OnHide(self)
	self:UnregisterEvent("PLAYER_SPECIALIZATION_CHANGED");
end

function BobUI_TalentFrameSpec_OnEvent(self, event, ...)
	if event == "PLAYER_SPECIALIZATION_CHANGED" and VIEWED_SPELL_BOOK ~= nil then
		VIEWED_SPELL_BOOK = 2
		VIEWED_SPELL_BOOK_NUM_SPELLS = GetNumSpellsForSpellBook(VIEWED_SPELL_BOOK)

		createSpellButtons()
		updateSpellButtons()
		specButtonsUpdateBorders()
	end
end





function SpecButton_OnShow(self)
	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);
	local currentSpecID = GetSpecialization()
	local cSpecID, currentSpecName = GetSpecializationInfo(currentSpecID)
	local sex = UnitSex("player");
	
	local roleNames = {
		["HEALER"] = HEALER,
		["DAMAGER"] = DAMAGER,
		["TANK"] = TANK
	}
	
	BobUI_TabTitle.specName:SetText(SPECIALIZATION .. ": " .. currentSpecName)

	local numTabs = 2 + numSpecs + (PetHasSpellbook() and 1 or 0) -- general tab & crafting tab + num specs + pet spells?

	self:Show();

	if self:GetID() == 1 then

			self.specIcon:SetTexture(136830)
			self.roleIcon:Hide();
			self.tooltip = GetSpellTabInfo(select(1, 1))

	elseif self:GetID() <= 1+numSpecs then
			local specID, specName, specDescription, specIcon, specRole, mainStat = GetSpecializationInfo(self:GetID()-1) 
			
			self.specIcon:SetTexture(specIcon);
			
			self.tooltip =  specName .. " " .. className ..  "\n\n" .. specDescription .. "\n\n" .. ROLE .. ": " .. roleNames[specRole];
			self.roleIcon:SetTexCoord(GetTexCoordsForRole(specRole));
	elseif self:GetID() > numSpecs and self:GetID() < 6 then
			self:Hide();
	
	elseif self:GetID() == 6 then
		local hasPetSpells, petToken = HasPetSpells();

		if ( hasPetSpells and PetHasSpellbook() ) then
			self.specIcon:SetTexture("Interface/Icons/ABILITY_HUNTER_SICKEM");
			self.roleIcon:Hide();
		else
			if VIEWED_SPELL_BOOK == 6 then VIEWED_SPELL_BOOK = 2 end
			self:Hide();
		end
	elseif self:GetID() == 7 then
			self.specIcon:SetTexture("Interface/Icons/TRADE_ARCHAEOLOGY_OGRE2HANDEDHAMMER");
			self.roleIcon:Hide();
			self.tooltip = TRADE_SKILLS
	elseif self:GetID() == 8 then
			self.specIcon:SetTexture("Interface/Icons/INV_Jewelry_Trinket_16");
			self.roleIcon:Hide();
			self.tooltip = TALENTS
	end

		VIEWED_SPELL_BOOK_NUM_SPELLS = GetNumSpellsForSpellBook(VIEWED_SPELL_BOOK)

		createSpellButtons()
		updateSpellButtons()
		specButtonsUpdateBorders()

end

function SpecButton_OnDoubleClick(self) 
	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);
	if self:GetID() < 2 or self:GetID() > numSpecs+1 or self:GetID()-1 == GetSpecialization() then return end -- can't change specialization to general spell book or pet/profession. 

	VIEWED_SPELL_BOOK = self:GetID()
	SetSpecialization(self:GetID()-1)
end

function SpecButton_OnClick(self)
	VIEWED_TAB_ID = self:GetID()
	
	if self:GetID() == 8 then
		BobUI_toggleTabPage(BobTabPage2)
	else
		BobUI_toggleTabPage(BobTabPage1)
		local className, classTag, classID = UnitClass("player");
		local numSpecs = GetNumSpecializationsForClassID(classID);

		specs = {}

		table.insert(specs, 1)

		for i=2, numSpecs+1 do -- current specialization is always number 2 spell book.. so need to rearrange books
			if (i < (GetSpecialization()+1)) then
				table.insert(specs, i + 1)
			elseif (i > (GetSpecialization()+1)) then
				table.insert(specs, i)
			else
				table.insert(specs, 2)
			end
		end

		if self:GetID() <= 5 then 
			VIEWED_SPELL_BOOK = specs[self:GetID()]
		else 
			VIEWED_SPELL_BOOK = self:GetID()
		end

		VIEWED_SPELL_BOOK_NUM_SPELLS = GetNumSpellsForSpellBook(VIEWED_SPELL_BOOK)
		CURRENT_FLYOUT = nil

		createSpellButtons()
		updateSpellButtons()
		updateSpellBookLabel()
	end

	specButtonsUpdateBorders()
end

function updateSpellBookLabel()
	if VIEWED_SPELL_BOOK < 6 then
		BobTabPage1Actives.category:SetText(GetSpellTabInfo(VIEWED_SPELL_BOOK))
	elseif VIEWED_SPELL_BOOK == 6 then
		BobTabPage1Actives.category:SetText(PET)
	else
		--BobTabPage1Actives.category:SetText(TRADE_SKILLS)
		BobTabPage1Actives.category:SetText()
	end
end

function SpecButton_OnEnter(self)
	if ( not self.selected ) then
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR");
		GameTooltip:AddLine(self.tooltip, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		if self:GetID() > 1 and self:GetID() < 5 then
			GameTooltip:SetMinimumWidth(300, true);
		end

		GameTooltip:Show();
	end
end

function SpecButton_OnLeave(self)
	GameTooltip:SetMinimumWidth(0, false);
	GameTooltip:Hide();
end

function BobUI_AbilityTab_OnEvent(self, event)
	if not InCombatLockdown() then
		if event == "PLAYER_REGEN_DISABLED" then
			if BobUI_AbilityTab:IsShown() then
				SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT = true
				BobUI_AbilityTab:Hide()
			else
				SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT = false
			end
		elseif event == "PLAYER_REGEN_ENABLED" then
			if SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT then
			BobUI_AbilityTab:Show()
			end
		else
			
			if (event == "SPELLS_CHANGED") then
				SpecButton_OnShow(BobUI_specButtons.specButton6)
				createSpellButtons()
				updateSpellButtons()
				specButtonsUpdateBorders()
			end
		end
	end
end

function CloseButton_OnEnter(self)
	self.close:SetVertexColor(0.6, 0.1, 0.1, 0.8)

end
function CloseButton_OnLeave(self)
	self.close:SetVertexColor(0.8, 0.1, 0.1, 0.8)
end
function CloseButton_OnShow(self)
	self.close:SetVertexColor(0.8, 0.1, 0.1, 0.8)
end


function CliqueShown()
	if CliqueConfig ~= nil then
		return CliqueConfig:IsShown()
	end

	return false
end


function BobUI_AbilityTab_OnLoad()
	tinsert(UISpecialFrames, "BobUI_AbilityTab")
	HideFrames:Hide()

	SpellBookFrame:SetParent(HideFrames)

	BobUI_AbilityTab:RegisterEvent("PLAYER_REGEN_DISABLED")
	BobUI_AbilityTab:RegisterEvent("PLAYER_REGEN_ENABLED")

	SpellBookFrame:SetScript("OnShow", function()
		if CliqueShown() == false then
			if SpellBookFrame:GetParent() ~= HideFrames then SpellBookFrame:SetParent(HideFrames) end
			SpellBookFrame:UnregisterAllEvents();
		end
	end)
	

	hooksecurefunc("TalentFrame_LoadUI", function()
		PlayerTalentFrame:SetParent(HideFrames)
		PlayerTalentFrame:UnregisterAllEvents();
	end)

	hooksecurefunc("ShowUIPanel", function(frame)
		local frameName = frame
		if type(frame) == "table" then frameName = frame:GetName() end

		if frame ~= nil then
			if ((frameName == "SpellBookFrame" and CliqueShown() == false) or (frameName == "PlayerTalentFrame")) then
				if frame:IsShown() then HideUIPanel(frame) end
				toggleSpellBook()
			else
				if frameName == "SpellBookFrame" then
					SpellBookFrame:SetParent(UIParent)
					SpellBookFrame.bookType = BOOKTYPE_SPELL;
					SpellBookFrame_Update();
				end
			end
		end

	end)

	if VIEWED_SPELL_BOOK == nil then
		VIEWED_SPELL_BOOK = 2
		VIEWED_TAB_ID = GetSpecialization() + 1
	end
	
	BobUI_Main:SetParent(UIParent)
	BobUI_AbilityTab:SetParent(BobUI_Main)
	BobUI_Main:SetScale(GPPS())
	BobUI_AbilityTab:SetFrameLevel(2)
	BobUI_AbilityTab:SetFrameStrata("HIGH")
	BobUI_Main:Show()

	BobUI_toggleTabPage(BobTabPage1)
end

function BobUI_toggleTabPage(shown)
	shown:Show()
	local hidden = nil

	if shown == BobTabPage1 then hidden = BobTabPage2 else hidden = BobTabPage1 end

	hidden:Hide()
end


function GPPS()
	local scale = string.match( GetCVar( "gxWindowedResolution" ), "%d+x(%d+)" );
	local uiScale = UIParent:GetScale(); 

	return 768/scale/uiScale;
end

function BobUI_AbilityTab_OnShow(self)
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("CURRENT_SPELL_CAST_CHANGED");
	self:RegisterEvent("TRADE_SKILL_SHOW");
	self:RegisterEvent("TRADE_SKILL_CLOSE");
	self:RegisterEvent("ARCHAEOLOGY_CLOSED");
	self:RegisterEvent("PET_BAR_UPDATE");
	self:RegisterEvent("CURSOR_UPDATE");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	
	SpecButton_OnShow(BobUI_specButtons.specButton6)
	local prof1, prof2, arch, fish, cook = GetProfessions();
	BobUI_FormatProfession(1, prof1);
	updateSpellBookLabel()
end

function BobUI_AbilityTab_OnHide(self)
	self:UnregisterEvent("SPELLS_CHANGED");
	self:UnregisterEvent("SPELL_UPDATE_COOLDOWN");
	self:UnregisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:UnregisterEvent("CURRENT_SPELL_CAST_CHANGED");
	self:UnregisterEvent("TRADE_SKILL_SHOW");
	self:UnregisterEvent("TRADE_SKILL_CLOSE");
	self:UnregisterEvent("ARCHAEOLOGY_CLOSED");
	self:UnregisterEvent("PET_BAR_UPDATE");
	self:UnregisterEvent("CURSOR_UPDATE");
	self:UnregisterEvent("ACTIONBAR_SLOT_CHANGED");

end

function toggleSpellBook()
    if InCombatLockdown() then
		return
	else
		if BobUI_AbilityTab:IsShown() then BobUI_AbilityTab:Hide() else BobUI_AbilityTab:Show() end
	end
end

